#!/usr/bin/python3

from flask import Flask, url_for, request, Blueprint
from authlib.integrations.flask_client import OAuth
from authlib.oidc.core import CodeIDToken
from authlib.jose import jwt
import pprint
import html
import os
import requests
import sys


app = Flask(__name__)

app.secret_key = "mysecretkey"

oauth = OAuth(app)

ID_BROKER_KEYCLOAK_FQDN = os.environ.get("ID_BROKER_KEYCLOAK_FQDN")
CLIENT_ID = os.environ.get("CLIENT_ID")
CLIENT_SECRET = os.environ.get("CLIENT_SECRET")
APPLICATION_ROOT = os.environ.get("APPLICATION_ROOT", "")
SCHEME = os.environ.get("SCHEME", "HTTP")

bp = Blueprint('uta', __name__)


if not ID_BROKER_KEYCLOAK_FQDN or not CLIENT_ID or not CLIENT_SECRET:
    raise ValueError("Not all env set, check: ID_BROKER_KEYCLOAK_FQDN, CLIENT_ID, CLIENT_SECRET")


univention_kc = oauth.register(
    name="univention_kc",
    client_id=CLIENT_ID,
    client_secret=CLIENT_SECRET,
    code_challenge_method="S256",
    scope="openid",  # This returns an id token
    server_metadata_url=f"https://login.{ID_BROKER_KEYCLOAK_FQDN}/auth/realms/ID-Broker/.well-known/openid-configuration"
)


def check_connection():
    try:
        requests.get(f"https://login.{ID_BROKER_KEYCLOAK_FQDN}")
    except Exception as exc:
        print(f"Could not reach https://login.{ID_BROKER_KEYCLOAK_FQDN}")
        print("Check if it can be resolved and add it to /etc/hosts if necessary")
        print("Check that the CA is available.")
        print("You can get it from \"http://$PRIMARY/ucs-root-ca.crt\"")
        print("Use the env \"REQUESTS_CA_BUNDLE\" and set it to the downloaded CA")
        print(exc)
        sys.exit(1)


@bp.route("/")
def saludo():
    kc_idp_hint = html.escape(request.args.get("kc_idp_hint", ""), quote=True)
    return f"""
Hello<br>
    <form action={url_for(".login")}>
        <label>From which school do you want to login?
            <input type="text" name="kc_idp_hint" value="{kc_idp_hint}">
        </label>
        <button id=login>"Login"</button>
    </form>
"""


@bp.route("/login")
def login():
    kc_idp_hint = request.args.get("kc_idp_hint", None)
    if not kc_idp_hint:
        raise Exception("No kc_idp_hint request parameter")
    redirect_url = url_for(".authorize", _external=True, _scheme=SCHEME)
    return univention_kc.authorize_redirect(redirect_url, kc_idp_hint=kc_idp_hint)


@bp.route("/authorize")
def authorize():
    # ONLY EVER RETURN THE TOKEN FOR DEBUGGING!!!
    auth_resp = univention_kc.authorize_access_token()
    id_claims = jwt.decode(auth_resp['id_token'], univention_kc.fetch_jwk_set(), claims_cls=CodeIDToken)
    id_claims.validate()
    return f"""
Success<br>
Pseudonym:
<p id=pseudonym>{id_claims['username']}</p>
Tokens FOR DEBUGGING ONLY!<br>
Token endpoint response:<br>
<pre style="white-space: pre-wrap; word-wrap: break-word;">{html.escape(pprint.pformat(auth_resp), quote=True)}</pre><br>
Encoded access Token:<br>
<pre style="white-space: pre-wrap; word-wrap: break-word;">{html.escape(pprint.pformat(auth_resp["access_token"]), quote=True)}</pre><br>
Decoded access token<br>
<pre style="white-space: pre-wrap; word-wrap: break-word;">{html.escape(pprint.pformat(jwt.decode(auth_resp["access_token"], key=univention_kc.fetch_jwk_set())), quote=True)}</pre><br>
Encoded id token:<br>
<pre style="white-space: pre-wrap; word-wrap: break-word;">{html.escape(pprint.pformat(auth_resp['id_token']), quote=True)}</pre><br>
Decoded id token:<br>
<pre style="white-space: pre-wrap; word-wrap: break-word;">{html.escape(pprint.pformat(id_claims), quote=True)}</pre><br>
"""


if __name__ == "__main__":
    check_connection()
    app.register_blueprint(bp, url_prefix=APPLICATION_ROOT)
    app.run(debug=True, port=5000, host="0.0.0.0")
